(function () {
    // String Extend.
    String.prototype.format = function () {
        var args = arguments;
        return this.replace(/{(\d+)}/g, function (match, number) {
            return typeof args[number] != 'undefined' ? args[number] : match;
        });
    };

    // Storage.
    jQuery.support.sessionStorage = (('sessionStorage' in window) && window['sessionStorage'] !== null);

    jQuery.storage = {
        get: function (key) {
            if (jQuery.support.sessionStorage)
                return sessionStorage[key];
            else
                throw 'Session Storage Not Supported.';
        },

        set: function (key, value) {
            if (jQuery.support.sessionStorage)
                sessionStorage[key] = value;
            else
                throw 'Session Storage Not Supported.';
        }
    };

    // ATA Namespace.
    // Constants.
    var KEY_CURRENT_ITEM_NO = 'cn';    // 当前试题号。
    var KEY_START_TIME = 'st';    // 测试开始时间。
    var KEY_END_TIME = 'et';    // 测试结束时间。
    var kEY_ITEM_MARK_TEMPLATE = 'im-{0}';    // 试题标记状态。
    var kEY_ITEM_COMPLATE_TEMPLATE = 'ic-{0}';    // 试题完成状态。

    if (!window.ATA) {
        window['ATA'] = {
            // Fields.
            _endTime: null,

            // Properties.
            // Get or set current item number.
            getCurrentItemNo: function () {
                return parseInt(jQuery.storage.get(KEY_CURRENT_ITEM_NO)) || 1;
            },

            setCurrentItemNo: function (itemNo) {
                jQuery.storage.set(KEY_CURRENT_ITEM_NO, itemNo);
            },

            // Get count of items.
            getItemCount: function () {
                return window.paper2.length;
            },

            getSubItemCount: function () {
                return window.paper2count || 0;
            },

            // Set the time of beginning.
            setStartTime: function (time) {
                jQuery.storage.set(KEY_START_TIME, time);
                this._endTime = time + this.getDuration();
                jQuery.storage.set(KEY_END_TIME, this._endTime);
            },

            // Get the time of ending.
            getEndTime: function () {
                return this._endTime ? this._endTime : (parseInt(jQuery.storage.get(KEY_END_TIME)) || 0);
            },

            // Get duration of the exam (ms).
            getDuration: function () {
                return window.paper.duration || 0;
            },

            // Get current time.
            getNow: function () {
                return new Date().getTime();
            },

            // Methods.
            // Ensure.
            ensure: function () {
                if (!window.paper)
                    throw '试题结构不存在。';
                if (!window.paper.items || window.paper.items.length == 0)
                    throw '试题信息不存在。';
                var o = ATA.loadItemInfo();
                window.paper2 = o.paper;
                window.paper2count = o.count;

            },

            // Get information of item.
            getItemInfo: function (itemNo) {
                var i = itemNo - 1;
                if (i > -1 && i < window.paper2.length)
                    return window.paper2[i];
            },

            loadItemInfo: function () {
                var paper2 = [],
                p = window.paper.items;
                var x = 0;
                for (var i = 0; i < p.length; i++) {
                    var t = '{0}、{1}'.format(['一', '二', '三', '四'][i], p[i].type);
                    for (var j = 0; j < p[i].detail.length; j++) {
                        x += p[i].detail[j].range.length;
                        paper2.push({
                            type: t,
                            range: p[i].detail[j].range.length == 1 ?
							'{0}'.format(p[i].detail[j].range[0]) :
							'{0}-{1}'.format(p[i].detail[j].range[0], p[i].detail[j].range[p[i].detail[j].range.length - 1]),
                            typestr: p[i].type
                        });
                    }
                }
                return {
                    paper: paper2,
                    count: x
                };
            },


            // Get or set mark flag of item, true for mark, false for unmark.
            getItemMark: function (itemNo) {
                return parseInt(jQuery.storage.get(kEY_ITEM_MARK_TEMPLATE.format(itemNo))) ? true : false;
            },

            setItemMark: function (itemNo, status) {
                jQuery.storage.set(kEY_ITEM_MARK_TEMPLATE.format(itemNo), status ? '1' : '0');
            },

            // Get or set complate flag of item, true for compated, false for uncomplated.
            getItemComplate: function (itemNo) {
                return parseInt(jQuery.storage.get(kEY_ITEM_COMPLATE_TEMPLATE.format(itemNo))) ? true : false;
            }
        };

        ATA.ensure();
    }

    if (!ATA.client) {
        ATA.client = {
            // Fields.
            _intervalId: null,

            // Methods.
            // Init client.
            init: function () {
                if (ATA.getEndTime() == 0) {
                    // Start exam.
                    ATA.setStartTime(ATA.getNow());
                }

                this.showRemainTime();
                // Countdown.
                this._intervalId = setInterval(function () {
                    ATA.client.showRemainTime();
                }, 1000);

                this.goto();
            },
            closeUserInfo: function () {
                document.getElementById('wrap').style.display = 'none';
            },
            closeXishuInfo: function () {
                document.getElementById('wrap_modulus').style.display = 'none';
            },
            showRequirment: function (id) {
                if (id == 1) {
                    jQuery('#tab_1').css('display', '');
                    jQuery('#tab_2').css('display', 'none');
                    jQuery('#tab_3').css('display', 'none');
                }
                if (id == 2) {
                    jQuery('#tab_1').css('display', 'none');
                    jQuery('#tab_2').css('display', '');
                    jQuery('#tab_3').css('display', 'none');
                }
                if (id == 3) {
                    jQuery('#tab_1').css('display', 'none');
                    jQuery('#tab_2').css('display', 'none');
                    jQuery('#tab_3').css('display', '');
                }
            },
            showXishuInfo: function () {
                jQuery('#wrap_modulus').css('display', 'block');
            },

            // Redirect to item page.
            redirect: function (itemNo) {
                //console.log(itemNo);
                if (itemNo)
                    ATA.setCurrentItemNo(itemNo);
                location.href = 'client.html';
            },

            // Goto the item.
            goto: function (itemNo) {
                var no = itemNo ? itemNo : ATA.getCurrentItemNo();

                // Update current item number.
                if (itemNo)
                    ATA.setCurrentItemNo(itemNo);

                // Update information of item.
                var count = ATA.getItemCount();

                this.innerStrPro(no);
                //jQuery('#progress').html('第<s>{0}</s>题'.format(ATA.getItemInfo(no).range));

                // Show or hide previous & next button.
                if (no > 0)
                    jQuery('#prev').css('visibility', 'visible');
                else
                    jQuery('#prev').css('visibility', 'hidden');

                if (no < count)
                    jQuery('#next').css('visibility', 'visible');
                else
                    jQuery('#next').css('visibility', 'hidden');

                // Show item.
                jQuery('iframe').attr('src', 'items/{0}.html'.format(no));

                // Update mark status.
                jQuery('#mark').attr('checked', ATA.getItemMark(no));
            },
            innerStrPro: function (no) {
                var p = window.paper.items;
                for (var i = 0; i < p.length; i++) {
                    if (p[i].type == ATA.getItemInfo(no).typestr) {
                        if (p[i].detail.length == 1) {
                            jQuery('#type').text('{0}{1}'.format(ATA.getItemInfo(no).type, ''));
                            jQuery('#progress').html('');
                        } else {
                            jQuery('#type').text('{0}{1}'.format(ATA.getItemInfo(no).type, ATA.getItemInfo(no).range ? '：' : ''));
                            jQuery('#progress').html('第<s>{0}</s>题'.format(ATA.getItemInfo(no).range));
                        }
                    }
                }
            },
            // Goto next item.
            next: function () {
                var no = ATA.getCurrentItemNo();
                if (no < ATA.getItemCount())
                    this.goto(++no);
            },

            // Goto previous item.
            previous: function () {
                var no = ATA.getCurrentItemNo();
                if (no == 1)
                    location.href = 'summary.html';
                if (no > 1)
                    this.goto(--no);
            },

            // Mark or unmark item.
            mark: function () {
                ATA.setItemMark(ATA.getCurrentItemNo(), jQuery('#mark').attr('checked'));
            },

            // End exam.
            end: function () {
                if (this.confirm('请确认：现在交卷吗？\n按“确定”将结束考试\n按“否”可继续考试')) {
                    // 1st.
                    if (this.confirm('请再次确认：现在交卷吗？\n按“确定”将结束考试\n按“否”可继续考试')) {
                        // 2nd.
                        if (this.confirm('请最后确认：现在交卷吗？\n按“确定”将结束考试\n按“否”可继续考试')) {
                            // Final(3rd)
                            location.href = 'end.html';
                        }
                    }
                }
            },

            /*// Clear all settings.
            clear: function () {
                if (jQuery.support.sessionStorage)
                    sessionStorage.clear();
                else
                    throw 'Session Storage Not Supported.'
            },*/

            // Display remain time.
            showRemainTime: function () {
                var t = Math.floor((ATA.getEndTime() - ATA.getNow()) / 1000);
                if (t < 1) {
                    // Game over.
                    clearInterval(this._intervalId);
                    location.href = 'end.html';
                }
                else {
                    var s = t % 60;
                    t = Math.floor(t / 60);
                    var t2 = ATA.client.prompt.getTime();
                    if (t < t2)
                        ATA.client.prompt.update();
                    if (t == t2 && s < 2) {
                        var msg = ATA.client.prompt.getMessage();
                        ATA.client.prompt.update();
                        this.alert(msg);
                    }
                    var m = t % 60;
                    var h = Math.floor(t / 60);

                    jQuery('#countdown').text('0{0}:{1}:{2}'.format(h, m < 10 ? '0' + m : m, s < 10 ? '0' + s : s));
                }
            },

            // Display information of candidate.
            showCandidateInfo: function () {
                jQuery('#wrap').css('display', 'block');
		ATA.client.showDivCenter('wrap','wrap_div');
            },

            showXishuInfo: function () {
                jQuery('#wrap_modulus').css('display', 'block');
		ATA.client.showDivCenter('wrap_modulus','wrap_xishu_div');
            },
	showDivCenter:function(wrap,wrap_div){
		var windowWidth = document.getElementById(wrap).clientWidth;
		var popupWidth = $("#"+wrap_div).width();
		$("#"+wrap_div).css({
			"position": "absolute",
			"left": windowWidth/2-popupWidth/2
		});
	},

            // Display preview details.
            showPreviewDetails: function () {
                var n = parseInt(jQuery(this).attr('section'));
                if (n && typeof n === 'number') {
                    jQuery('#tab li').removeClass('cur').each(function (i, el) {
                        if (n - 1 == i)
                            $(el).addClass('cur');
                    });
                    jQuery('#details table').hide().each(function (i, el) {
                        if (n - 1 == i)
                            $(el).css('display', '');
                    });
                }
            },

            // Alert.
            alert: function (message) {
                if (message)
                    window.alert(message);
            },

            // Confirm.
            confirm: function (message) {
                if (message)
                    return window.confirm(message);
                return false;
            }
        };
    }

    if (!ATA.client.prompt) {
        ATA.client.prompt = {
            // Fields.
            _prompt: null,
            _prompts: null,

            // Properties.
            // Get the last time (minute).
            getTime: function () {
                if (this._prompt) {
                    return this._prompt.time;
                }
                return -1;
            },

            // Get the message of last time.
            getMessage: function () {
                if (this._prompt) {
                    return this._prompt.message;
                }
                return null;
            },

            // Methods.
            // Init.
            init: function () {
                var p = window.paper.prompts;
                if (p && jQuery.isArray(p) && p.length > 0) {
                    // init prompt array.
                    this._prompts = [];
                    for (var i = 0; i < p.length; i++) {
                        this._prompts.push(p[i]);
                    }

                    // Sort.
                    this._prompts.sort(function (x, y) { return x.time - y.time; });

                    // Load first prompt.
                    this._prompt = this._prompts.pop();
                }
            },

            // Update the last prompt.
            update: function () {
                if (this._prompts && this._prompts.length > 0)
                    this._prompt = this._prompts.pop();
                else
                    this._prompt = null;
            }
        };

        ATA.client.prompt.init();
    }

    if (!ATA.client.preview) {
        ATA.client.preview = {
            // Init preview page.
            init: function () {
                // Generate body.
                var p = window.paper.items;
                var html = '';
                var x = 0;
                for (var i = 0; i < p.length; i++) {
                    html += '<dl><dt>{0}</dt><dd>'.format(p[i].type);
                    for (var j = 0; j < p[i].detail.length; j++) {
                        for (var k = 0; k < p[i].detail[j].range.length; k++) {
                            html += '<a href="#" item="{0}" sub="{1}">{2}</a>'.format(p[i].detail[j].no, ++x, p[i].detail[j].range[k]);
                        }
                    }
                    html += '</dd></dl>';
                }
                jQuery('#details p').after(html);

                var ln = ATA.getSubItemCount() + 1;
                for (var i = 1; i < ln; i++) {
                    if (ATA.getItemComplate(i))
                        jQuery('#details a[sub={0}]'.format(i)).addClass('yes');
                    else
                        jQuery('#details a[sub={0}]'.format(i)).removeClass('yes');
                }
                ln = ATA.getItemCount() + 1;
                for (var i = 1; i < ln; i++) {
                    if (ATA.getItemMark(i))
                        jQuery('#details a[item={0}]'.format(i)).addClass('mark');
                    else
                        jQuery('#details a[item={0}]'.format(i)).removeClass('mark');
                }
            }
        };
    }
})();
